/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   This program creates an animation by repeatedly saving the framebuffer's image to a file.
   <p>
   You can see the animation by quickly sequencing through the frame images.
*/
public class OfflineMovie
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render
      final Scene scene = new Scene();

      // Create several Model objects.
      final Model m1 = new ObjSimpleModel(new File("../assets/apple.obj"));
      final Model m2 = new ObjSimpleModel(new File("../assets/cow.obj"));
      final Model m3 = new ObjSimpleModel(new File("../assets/cessna.obj"));
      final Model m4 = new ObjSimpleModel(new File("../assets/galleon.obj"));
      final Model m5 = new ObjSimpleModel(new File("../assets/teapot.obj"));
      final Model m6 = new ObjSimpleModel(new File("../assets/small_rhombicosidodecahedron.obj"));
      final Model m7 = new ObjSimpleModel(new File("../assets/great_rhombicosidodecahedron.obj"));

      // Add Models to the Scene.
      scene.addModel(m1, m3, m4, m2);//apple, cessna, galleon, cow

      // Push the models away from where the camera is.
      for (final Model m : scene.modelList)
      {
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x,
                                           v.y,
                                           v.z - 3));
         }
      }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a framebuffer to render our scene into.
      final int width  = 1024;
      final int height = 1024;
      final FrameBuffer fb = new FrameBuffer(width, height);
      // Give the framebuffer a nice background color.
      fb.clearFB(Color.darkGray);

      Rasterize_Clip.doClipping = false; // turn off clipping

      // Render our scene into the frame buffer.
      Pipeline.render(scene, fb.vp);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_OfflineModels_Frame%03d.ppm", 0) );

      for (int j = 1; j <= 100; ++j)
      {
         // Update the scene.
         Model m = scene.modelList.get(0);
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x + 0.3,   // move right
                                           v.y + 0.1,   // move up
                                           v.z - 0.1)); // move back
         }
         m = scene.modelList.get(1);
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x - 0.3,   // move left
                                           v.y - 0.1,   // move down
                                           v.z - 0.1)); // move back
         }
         m = scene.modelList.get(2);
         for (int i = 0; i < m.vertexList.size(); ++i)
         {
            final Vertex v = m.vertexList.get(i);
            m.vertexList.set(i, new Vertex(v.x + 0.01, // move right
                                           v.y,
                                           v.z));
         }

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb.vp);
         fb.dumpFB2File(String.format("PPM_OfflineModels_Frame%03d.ppm", j));
      }
   }
}
